<?php
/**
 * Class Nutriente
 *
 * PHP version 5
 *
 * @category Php
 * @package  Geral\Logic
 * @author   Erika Souza <erika.souza@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Geral\Logic;
use Geral\Util\AcumuladorDeMensagens;
use Geral\Util\DateUtil;
use Geral\Util\Ambiente;
use \Geral\Entities\Paramgeral;
use Planejamento\Entities\Grupnutr;
use \Geral\Entities\Nutriente as Nutri;
use Setforms2\Shared\Registry;
use Setforms2\Runtime\Framework;
use \Geral\Entities\Produto as Produto;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;

/**
 * Class Nutriente
 *
 * PHP version 5
 *
 * @category Php
 * @package  Geral\Logic
 * @author   Erika Souza <erika.souza@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Nutriente
{
    const EXPORTA_NUTRIENTES = 'EXPORTA_NUTRIENTES';
    /**
     * Deleta nutriente
     *
     * @param String $nutriente Nutriente
     * @param String $nrorg Organização
     *
     * @return null
     * @throws \Exception
     */
    public static function deletaNutriente($nutriente, $nrorg)
    {
        try {
            /** @var $objNutriente \Geral\Entities\Nutriente */
            $objNutriente = \Geral\Entities\Nutriente::getObject(
                array('cdnutriente' => $nutriente, 'nrorg' => $nrorg)
            );

            if (is_a($objNutriente, '\Geral\Entities\Nutriente')) {
                $objNutriente->delete();
                Registry::getEntityManager()->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o maior código de nutriente
     *
     * @param String $nrorg Organização
     *
     * @return string
     */
    public static function retornaMaxGrupoNutri($nrorg)
    {
        /** @var $GrupNutriRepositories \Planejamento\Repositories\Grupnutr */
        $GrupNutriRepositories = Grupnutr::getRepository();
        $max = $GrupNutriRepositories->retornaMaxGrupoNutri($nrorg);
        return str_pad(++$max, 2, 0, STR_PAD_LEFT);
    }


    public static function excluiGrupoNutri($grupo, $nrorg, $oper)
    {
        $nutrientesGrupo = Nutri::getObjectsByWhere(
            'NRORG = ? AND CDGRUPNUTR = ?',
            array($nrorg, $grupo)
        );

        $data = DateUtil::getDataAtual();
        $con = Registry::getEntityManager();
        $con->beginTransaction();

        /** @var $nutriente Nutri */
        foreach ($nutrientesGrupo as $nutriente) {
            $nutriente->atualizaGrupo(null, $data, $oper, $nrorg);
            $con->flush();
        }

        /** @var $dadosGrupo Grupnutr */
        $dadosGrupo = Grupnutr::getObject(
            array('nrorg' => $nrorg, 'cdgrupnutr' => $grupo)
        );

        $dadosGrupo->delete();
        $con->flush();
        $con->commit();

        return true;
    }

    /**
     * Atualiza nutriente
     *
     * @param String $nutriente Nutriente
     * @param String $grupo     Grupo
     * @param String $nrorg     Organização
     * @param String $oper      Operador
     *
     * @return bool
     */
    public static function atualizaNutriente($nutriente, $grupo, $nrorg, $oper)
    {
        /** @var $dadosNutriente Nutri */
        $dadosNutriente = Nutri::getObject(
            array('nrorg' => $nrorg, 'cdnutriente' => $nutriente)
        );

        if (!empty($dadosNutriente)) {
            $data = DateUtil::getDataAtual();
            $con = Registry::getEntityManager();
            $con->beginTransaction();

            $dadosNutriente->atualizaGrupo($grupo, $data, $oper, $nrorg);
            $con->flush();
            $con->commit();

            return true;
        }
        return false;
    }

    /**
     * Valida código
     *
     * @param String $codigo Código
     * @param String $nrorg  Organização
     *
     * @return bool
     */
    public static function validaCodigo($codigo, $nrorg)
    {
        /** @var $sql GrupNutr */
        $buscaCodigo = Grupnutr::getObject(
            array('nrorg' => $nrorg, 'cdgrupnutr' => $codigo)
        );
        if (!empty($buscaCodigo)) {
            tk_abort(tk_get_message_by_name("msg_codigoGrupoExistente"));
        } else {
            return true;
        }
    }

    /**
     * Gera lista do grupo de nutriente
     *
     * @param String $cdGrupNutri Código do grupo de nutriente
     *
     * @return string
     */
    public static function geraListaGrupNutri($cdGrupNutri)
    {
        $stListaGrupNutr = '_';
        if (is_array($cdGrupNutri)) {
            foreach ($cdGrupNutri as $grupNutr) {
                $stListaGrupNutr .= $grupNutr['cdgrupnutr'] . '_';
            }
        }
        return $stListaGrupNutr;
    }


    /**
     * Verifica se há mais de um registro com o campo IDKCAL = 'S'
     *
     * @param String $nrorg
     *
     * @return bool
     */

    public static function salvaNutriente($nrorg)
    {
        $linhas = Framework::getInstance()->getDataset();
        $qtd_e_caloria = 0;
        $sql = "NRORG = ? AND IDKCAL = ? ";
        $array_sql = array($nrorg, 'S');

        foreach ($linhas as $chave => $valor) {
            $sql .= 'AND CDNUTRIENTE <> ? ';
            $array_sql[] = $valor['CDNUTRIENTE'];
            if ($valor['IDKCAL'] === 'S') {
                $qtd_e_caloria++;
            }
        }

        $obj_Nutriente = \Geral\Entities\Nutriente::getObjectsByWhere($sql, $array_sql);

        if (!empty($obj_Nutriente)) {
            $qtd_e_caloria += count($obj_Nutriente);
        }

        if ($qtd_e_caloria > 1) {
            return false;
        } else {
            foreach ($linhas as $chave => $valor2) {
                $data = DateUtil::getDataAtual();
                $cdnutriente = $valor2['CDNUTRIENTE'];
                $nmnutriente = $valor2['NMNUTRIENTE'];
                $nmreduzido = $valor2['NMREDUZIDO'];
                $idtiponutr = $valor2['IDTIPONUTR'];
                $sgunidade = $valor2['SGUNIDADE'];
                $cdgrupnutr = $valor2['CDGRUPNUTR'];
                $cdoper = $valor2['CDOPERINCLUSAO'];
                $idativo = $valor2['IDATIVO'];
                $idcalorias = $valor2['IDCALORIAS'];
                $idnutrrelev = $valor2['IDNUTRRELEV'];
                $vrfatornutr = $valor2['VRFATORNUTR'];
                $idkcal = $valor2['IDKCAL'];
                $nrordemnutr = $valor2['NRORDEMNUTR'];

                if (empty($vrfatornutr)) {
                    $vrfatornutr = 0.00;
                }
                if ($valor2['_STATE'] === 'I') {
                    \Geral\Entities\Nutriente::insereNutriente($nrorg, $data, $cdnutriente, $nmnutriente, $nmreduzido, $idtiponutr, $sgunidade, $cdgrupnutr,
                        $cdoper, $idativo, $idcalorias, $idnutrrelev, $vrfatornutr, $idkcal, $nrordemnutr);
                } else {
                    \Geral\Entities\Nutriente::atualizaNutriente($nrorg, $data, $cdnutriente, $nmnutriente, $nmreduzido, $idtiponutr, $sgunidade, $cdgrupnutr,
                        $cdoper, $idcalorias, $idnutrrelev, $vrfatornutr, $idkcal, $nrordemnutr);
                }
            }
        }
        return true;
    }


    public static function verificaCamposVazios() {
        $linhas = Framework::getInstance()->getDataset();
        foreach ($linhas as $chave => $valor2) {
            $cdnutriente = $valor2['CDNUTRIENTE'];
            $nmnutriente = $valor2['NMNUTRIENTE'];
            $nmreduzido = $valor2['NMREDUZIDO'];
            $idtiponutr = $valor2['IDTIPONUTR'];
            $sgunidade = $valor2['SGUNIDADE'];
            $cdgrupnutr = $valor2['CDGRUPNUTR'];

            if (empty($cdnutriente)) {
                tk_abort(tk_get_message_by_name('msg_informe_codigo_nutriente'));
            } elseif (empty($nmnutriente)) {
                tk_abort(tk_get_message_by_name('msg_informe_nome_nutriente'));
            } elseif (empty($nmreduzido)) {
                tk_abort(tk_get_message_by_name('msg_informe_sigla_nutriente'));
            } elseif (empty($sgunidade)) {
                tk_abort(tk_get_message_by_name('msg_informe_un_nutriente'));
            } elseif (empty($cdgrupnutr)) {
                tk_abort(tk_get_message_by_name('msg_informe_grupo_nutrientes'));
            } elseif (empty($idtiponutr)) {
                tk_abort(tk_get_message_by_name('msg_informe_tipo_nutriente'));
            }
        }
    }


    /**
     * Quando o parâmetro "Integra nutrientes da Teknisa" na tela de Parâmetros do Nutriente a tela de Nutriente,
     * Grupo de Nutriente e Nutriente por Produto devem ser desabilitadas, possibilitando apenas visualização.
     * @static
     * @return null
     * id 1007008
     */
    public static function verificaIntegracao($tabName) {
        /** @var $parametro Paramgeral */

        $nrorg = Ambiente::getNrorg();
        $parametro = Paramgeral::getObjectByWhere("NRORG = ?  " , array($nrorg));
        $id = $parametro->getIdintegra();
        if($id == 'S') {
            tk_disable_grid($tabName);
            tk_set_disable_action("tk_form_view", $tabName);
            if($tabName !== 'sit00200_000_001'){
                tk_set_disable_action("tk_new_row_u", $tabName);
                tk_set_disable_action("tk_delete_row_u", $tabName);
            }
        }
    }

    public static function updateProdutos($nrorg, $nivelmax, $produtos, $idnutriente, $alteranutriente, $observacao)
    {
        $listaProduto = '';
        $resultado    = null;
        $contador     = 0;

        $cdproduto = explode(',', $produtos);
        array_pop($cdproduto);
        $tamanho   = count($cdproduto);

        foreach ($cdproduto as $produto) {
            $listaProduto .= $produto . ",";
            $contador++;
           if($contador == $tamanho || $contador == 100) {
              if($tamanho > 100) {
                $tamanho = $tamanho - 100;
              }
                $resultado = self::atualizaProdutoBanco($nrorg, $nivelmax, $listaProduto, $idnutriente, $alteranutriente, $observacao);
                $listaProduto = '';
                $contador = 0;
           }

        }
        if($resultado == true) {
            return true;

        }
    }


    public static function atualizaProdutoBanco($nrorg, $nivelmax, $listaProduto, $idnutriente, $alteranutriente, $observacao) {

        $id           = 'S';

        /** @var $objetos Produto[] */

        $objetos = Produto::getObjectsByWhere('NRORG = (?)  AND   IDPRODATIVPR = (?) AND CDNVPRODUTO = (?) AND INSTR ((?) , CDARVPROD) <> 0 ', array($nrorg, $id,  $nivelmax, $listaProduto));

        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($objetos) === true && !empty($objetos)) {

                foreach ($objetos as $objeto) {
                    $objeto->atualizaIdentNutricional($idnutriente, $alteranutriente, $observacao);
                }
            }
            $conn->flush();
            $conn->commit();

            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }

    /**
     * Updata os fields da tela Merendas da window 02510 com null
     * @static
     * @return null
     * @throws \Exception
     * id 1007095 (Método que o chama)
     */
    public static function zeraCamposNutriente($nrorg, $cdpargeral)
    {
        /** @var $registro PARAMGERAL */
        $registro = PARAMGERAL::getObjectByWhere('NRORG = ? AND CDPARGERAL = ?', array($nrorg, $cdpargeral));

        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($registro) === true && !empty($registro)) {
                $registro->setaNullCamposNutriente();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }

    /**
     * Updata os fields da tela Merendas da window 02510 com null
     * @static
     * @return null
     * @throws \Exception
     * id 1007095 (Método que o chama)
     */

    public static function zeraCamposIntegracao($nrorg, $cdpargeral)
    {
        /** @var $registro PARAMGERAL */
        $registro = PARAMGERAL::getObjectByWhere('NRORG = ? AND CDPARGERAL = ?', array($nrorg, $cdpargeral));

        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($registro) === true && !empty($registro)) {
                $registro->setaNullCamposIntegracao();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }

    /**
     * Procedure que exporta os nutrientes para determinado produto
     * @static
     * @return null
     * @throws \Exception
     * id 1008188 (Método que o chama)
     */
    public static function executaProcedureNutr($prodOrigem, $listaNutrientes, $nrlistaProduto, $tipoOperacao)
    {

        $procedure = new Procedure ( self::EXPORTA_NUTRIENTES );
        $procedure->addParam ( new Param ( 'P_CDPRODORIG', Param::PARAM_INPUT, $prodOrigem, Param::PARAM_TYPE_STR ) );
        $procedure->addParam ( new Param ( 'P_LISTANUTR', Param::PARAM_INPUT, $listaNutrientes, Param::PARAM_TYPE_STR ) );
        $procedure->addParam ( new Param ( 'P_LISTAPROD', Param::PARAM_INPUT, $nrlistaProduto, Param::PARAM_TYPE_STR ) );
        $procedure->addParam ( new Param ( 'P_OPERACAO', Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR ) );

        $procedure->call ();

    }




}